/*
BUllCowGame.cpp includes definitions for all functions of BullCowGame class.
*/

#pragma once
#include "FBullCowGame.h"
#include <map>

// to make syntax Unreal friendly
#define TMap std::map 
using int32 = int;

FBUllCowGame::FBUllCowGame() { Reset(); }

int32 FBUllCowGame::GetCurrentTry() const { return MyCurrentTry; }
int32 FBUllCowGame::GetHiddenWordLength() const { return MyHiddenWord.length(); }

bool FBUllCowGame::IsGameWon() const { return bGameIsWon; }

FString FBUllCowGame::GetHintForHiddenWord() const
{
	TMap<FString, FString> HintList {
		{"cat", "it's a pet"},
		{"gold", "a thing that glitters" },
		{"drums", "a musical instrument"},
		{"family", "your closest people"},
		{"country", "a named piece of territory"}
	};
	return HintList[MyHiddenWord];
}


int32 FBUllCowGame::GetMaxTries() const 
{
	TMap <int32, int32> WordLengthToMaxTries{ {3,4}, {4,6}, {5,8}, {6,10}, {7,12} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

void FBUllCowGame::Reset()
{
	bGameIsWon = false;
	MyCurrentTry = 1;
	return;
}

EGuessStatus FBUllCowGame::CheckGuessValidity(FString Guess) const
{
	if (!IsHaveSpace(Guess))
	{
		return EGuessStatus::Have_Space;
	}
	if (!IsHavePunctuation(Guess))
	{
		return EGuessStatus::Have_Punctuation;
	}
	if (!IsHaveDigits(Guess))
	{
		return EGuessStatus::Have_Digits;
	}
	else if (Guess.empty())
	{
		return EGuessStatus::Is_Empty;
	}
	else if (!IsIsogram(Guess))// if guess isn't an isogram - error
	{
		return EGuessStatus::Not_Isogram; 
	}
	else if (!IsLowerCase(Guess)) // isn't lowercase  
	{
		return EGuessStatus::Not_Lowercase; 
	}
	else if ( GetHiddenWordLength() != Guess.length() ) // the guess length wrong
	{
		return EGuessStatus::Wrong_Length;
	}
	else
	{
		return EGuessStatus::OK;
	}	
}

// recieves a valid guess, increments turn, and returns count
FBullCowCount FBUllCowGame::SubmitValidGuess(FString GuessWord)
{	
	MyCurrentTry++;
	FBullCowCount BullCowCount;

	//loop through all letters in the hidden word
	int32 WordLength = MyHiddenWord.length();
	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++)
	{
		// compare lettters against the guess
		for (int32 GChar = 0; GChar < WordLength; GChar++)
		{
			// if they match then
			if (MyHiddenWord[MHWChar] == GuessWord[GChar])
			{
				//if the same place
				if (MHWChar == GChar)
				{					
					BullCowCount.Bulls++; // increment bulls
				}
				else
				{					
					BullCowCount.Cows++; // increment rement cows
				}
			}
		}
	}
	if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}
	else
	{
		bGameIsWon = false;
	}
	return BullCowCount;
}

bool FBUllCowGame::IsIsogram(FString Word) const
{
	TMap<char, bool> LetterSeen; // set up our map
	for (auto Letter : Word)	// for all letters of the word
	{
		Letter = tolower(Letter); // handle mixed case
		if (LetterSeen[Letter]) { return false; }
		else 
		{
			LetterSeen[Letter] = true;
		}		
	}
	return true; // for example in cases with numbers /0 is entered
}

bool FBUllCowGame::IsLowerCase(FString Word) const
{
	for (auto Letter : Word)
	{
		if (!islower(Letter))
		{
			return false;
		}				
	}
	return true;
}

bool FBUllCowGame::IsHaveDigits(FString Word) const
{
	for (auto Letter : Word)
	{
		if (isdigit(Letter))
		{
			return false;
		}
	}
	return true;
}

bool FBUllCowGame::IsHavePunctuation(FString Word) const
{
	for (auto Letter : Word)
	{
		if (ispunct(Letter))
		{
			return false;
		}
	}
	return true;
}

bool FBUllCowGame::IsHaveSpace(FString Word) const
{
	for (auto Letter : Word)
	{
		if (isspace(Letter)) return false;
	}
	return true;
}

void FBUllCowGame::InitializeWord(FString WordLength)
{
	TMap<FString, FString> WordList{ {"3","cat"}, {"4","gold"}, {"5", "drums"}, {"6","family"}, {"7", "country"} };
	MyHiddenWord = WordList[WordLength];
	return;
}
