/* This is the console executable that makes use of the BUllCow class
This acts as thw vies a MVC patter, and responsible foe all user interation.
For game logic see FBullCow class.
*/

#pragma once
#include <iostream>
#include <string>
#include "FBullCowGame.h"

// to make syntax Unreal friendly
using int32 = int;
using FText = std::string;

// function prototypes

void PrintIntro();
void PlayGame();
FText GetValidGuess();
bool AskToPlayAgain();
void PrintGameSummary();
void AskPlayerWordLength();
void PrintHint();


FBUllCowGame BCGame; // instantiate a new game, which we re-use

// the entry point for our application
int main()
{
	// introduce the game
	bool bPlayAgain = false;
	do {
		PrintIntro();
		AskPlayerWordLength();
		PlayGame(); 
		bPlayAgain = AskToPlayAgain();
	} while (bPlayAgain);
	return 0;
}

// plays single game until completion
void PlayGame()
{	
	BCGame.Reset();
	int32 MaxTries = BCGame.GetMaxTries();	
	// LOOP ASKING FOR GUESSES
	// while the game is NOT won and still tries remaining
	while (!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries)
	{
		FText Guess = GetValidGuess(); 
		//submit valid guedd to game
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);
		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << ". Cows = " << BullCowCount.Cows << "\n\n";
		if (BCGame.GetCurrentTry() == BCGame.GetMaxTries())
		{
			PrintHint();
		}
	}
	PrintGameSummary();
	return;
}

void PrintIntro()
{
	std::cout << "Welcome to Bulls and Cows, a fun word game.\n";
	std::cout << std::endl;
	std::cout << "          }   {         ___ " << std::endl;
	std::cout << "          (o o)        (o o) " << std::endl;
	std::cout << "   /-------\\ /          \\ /-------\\ " << std::endl;
	std::cout << "  / | BULL |O            O| COW  | \\ " << std::endl;
	std::cout << " *  |-,--- |              |------|  * " << std::endl;
	std::cout << "    ^      ^              ^      ^ " << std::endl;
	std::cout << "Can you guess the isogram I'm thinking of?\n";
	std::cout << std::endl;
	return;
}

// loop util user puts valid guess
FText GetValidGuess() 
{
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	FText Guess = "";
	do {
		// get a guess from player
		int32 CurrentTry = BCGame.GetCurrentTry();
		std::cout << "Try " << CurrentTry << " of " << BCGame.GetMaxTries() << ". Enter your guess: ";
		
		std::getline(std::cin, Guess);

		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status)
		{
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n";
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter a word without repeating letters.\n";
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "PLease write word in lowercase.\n";
			break;
		case EGuessStatus::Is_Empty:
			std::cout << "Please enter a word.\n";
			break;
		case EGuessStatus::Have_Digits:
			std::cout << "PLease enter word without digits.\n";
			break;
		case EGuessStatus::Have_Punctuation:
			std::cout << "Please enter a word without punctuation marks.\n";
			break;
		case EGuessStatus::Have_Space:
			std::cout << "Please enter a full word (without spaces).";
			break;
		default:
			// assume the guess is valid
			break;
		}
		std::cout << std::endl;
	} while (Status != EGuessStatus::OK); // keep loop until get no errors
	return Guess;
}

bool AskToPlayAgain()
{
	std::cout << "Do you want to play again? (y/n)";
	FText Response = "";
	std::getline(std::cin, Response);
	return (Response[0] == 'y') || (Response[0] == 'Y');
}

void PrintGameSummary()
{
	if (BCGame.IsGameWon())
	{
		std::cout << "Congratulations! You won this game!\n";
	}
	else
	{
		std::cout << "You failed! Game over!\n";
	}
	return;
}

void AskPlayerWordLength()
{
	std::cout << "Enter the length of word you want to play with. From 3 to 7 letters. \n";
	FString WordLength = "";
	std::getline(std::cin, WordLength);
	BCGame.InitializeWord(WordLength);
	std::cout << "You chose " << BCGame.GetHiddenWordLength() << " letter word.\n";
	std::cout << "GOOD LUCK!!!\n\n";
	return;
}

void PrintHint()
{
	std::cout << "Oooops it your last chance\n";
	std::cout << "Hint: " << BCGame.GetHintForHiddenWord() << std::endl;
	return;
}
